@@ -12,8 +12,6 @@
 #include "cdf_int.h"
 #include "namespace.h"
 
-extern int CDF_Fatal;
-extern int CDF_Verbose;
 extern int CDF_Debug;
 
 #if  defined  (HAVE_LIBNETCDF)
@@ -56,7 +54,8 @@ void cdf_create(const char *path, int cmode, int *ncidp)
   chunksizehint = 16777216; /* 16 MB */
 #endif
 
-  if ( cdiNcChunksizehint != CDI_UNDEFID ) chunksizehint = cdiNcChunksizehint;
+  if ( cdiNcChunksizehint != CDI_UNDEFID )
+    chunksizehint = (size_t)cdiNcChunksizehint;
 
   cdi_nc__create_funcp my_nc__create =
     (cdi_nc__create_funcp)namespaceSwitchGet(NSSWITCH_NC__CREATE).func;
@@ -84,7 +83,7 @@ int cdf_open(const char *path, int omode, int *ncidp)
   size_t chunksizehint = 0;
 
 #if  defined  (HAVE_LIBNC_DAP)
-  if ( memcmp(path, "http:", 5) == 0 ) dapfile = TRUE;
+  if ( strncmp(path, "http:", 5) == 0 || strncmp(path, "https:", 6) == 0 ) dapfile = TRUE;
 #endif
 
   if ( dapfile )
@@ -101,7 +100,8 @@ int cdf_open(const char *path, int omode, int *ncidp)
       /*
       if ( chunksizehint < ChunkSizeMin ) chunksizehint = ChunkSizeMin;
       */
-      if ( cdiNcChunksizehint != CDI_UNDEFID ) chunksizehint = cdiNcChunksizehint;
+      if ( cdiNcChunksizehint != CDI_UNDEFID )
+        chunksizehint = (size_t)cdiNcChunksizehint;
 
       /* FIXME: parallel part missing */
       status = nc__open(path, omode, &chunksizehint, ncidp);
@@ -148,6 +148,20 @@ void cdf_enddef(int ncid)
 }
 
 
+void cdf__enddef(const int ncid, const size_t hdr_pad)
+{
+  int status;
+  const size_t v_align   = 4UL; /* [B] Alignment of beginning of data section for fixed variables */
+  const size_t v_minfree = 0UL; /* [B] Pad at end of data section for fixed size variables */
+  const size_t r_align   = 4UL; /* [B] Alignment of beginning of data section for record variables */
+
+  /* nc_enddef(ncid) is equivalent to nc__enddef(ncid, 0, 4, 0, 4) */
+  status = nc__enddef(ncid, hdr_pad, v_align, v_minfree, r_align);
+
+  if ( status != NC_NOERR ) Error("%s", nc_strerror(status));
+}
+
+
 void cdf_sync(int ncid)
 {
   int status;
@@ -522,6 +536,19 @@ void  cdf_get_vara_text(int ncid, int varid, const size_t start[],
 }
 
 
+void  cdf_get_vara_uchar(int ncid, int varid, const size_t start[], const size_t count[], unsigned char *tp)
+{
+  int status;
+
+  status = nc_get_vara_uchar(ncid, varid, start, count, tp);
+
+  if ( CDF_Debug || status != NC_NOERR )
+    Message("ncid = %d varid = %d", ncid, varid);
+
+  if ( status != NC_NOERR ) Error("%s", nc_strerror(status));
+}
+
+
 void cdf_put_var_double(int ncid, int varid, const double *dp)
 {
   int status;
@@ -674,8 +701,7 @@ void cdf_put_att_text(int ncid, int varid, const char *name, size_t len,
   status = nc_put_att_text(ncid, varid, name, len, tp);
 
   if ( CDF_Debug || status != NC_NOERR )
-    Message("ncid = %d varid = %d att = %s text = %s",
-	    ncid, varid, name, tp);
+    Message("ncid = %d varid = %d att = %s text = %.*s", ncid, varid, name, (int)len, tp);
 
   if ( status != NC_NOERR ) Error("%s", nc_strerror(status));
 }
@@ -709,7 +735,7 @@ void cdf_put_att_double(int ncid, int varid, const char *name, nc_type xtype,
 }
 
 
-void cdf_get_att_text(int ncid, int varid, char *name, char *tp)
+void cdf_get_att_text(int ncid, int varid, const char *name, char *tp)
 {
   int status;
 
@@ -721,8 +747,22 @@ void cdf_get_att_text(int ncid, int varid, char *name, char *tp)
   if ( status != NC_NOERR ) Error("%s", nc_strerror(status));
 }
 
+void cdf_get_att_string(int ncid, int varid, const char *name, char **tp)
+{
+#if  defined  (HAVE_NETCDF4)
+  int status;
 
-void cdf_get_att_int(int ncid, int varid, char *name, int *ip)
+  status = nc_get_att_string(ncid, varid, name, tp);
+
+  if ( CDF_Debug || status != NC_NOERR )
+    Message("ncid = %d varid = %d name = %s", ncid, varid, name);
+
+  if ( status != NC_NOERR ) Error("%s", nc_strerror(status));
+#endif
+}
+
+
+void cdf_get_att_int(int ncid, int varid, const char *name, int *ip)
 {
   int status;
 
@@ -735,7 +775,7 @@ void cdf_get_att_int(int ncid, int varid, char *name, int *ip)
 }
 
 
-void cdf_get_att_double(int ncid, int varid, char *name, double *dp)
+void cdf_get_att_double(int ncid, int varid, const char *name, double *dp)
 {
   int status;
 
