@@ -47,7 +47,7 @@ int recordNewEntry(stream_t *streamptr, int tsID)
       if ( records == NULL )
 	{
           Message("recordSize = %d", recordSize);
-	  SysError("Allocation of record_tTABLE failed");
+	  SysError("Allocation of records failed");
 	}
 
       for ( i = 0; i < recordSize; i++ )
@@ -74,7 +74,7 @@ int recordNewEntry(stream_t *streamptr, int tsID)
       if ( records == NULL )
 	{
           Message("recordSize = %d", recordSize);
-	  SysError("Reallocation of record_tTABLE failed");
+	  SysError("Reallocation of records failed");
 	}
       recordID = recordSize/2;
 
@@ -82,7 +82,6 @@ int recordNewEntry(stream_t *streamptr, int tsID)
 	records[i].used = CDI_UNDEFID;
     }
 
-
   recordInitEntry(&records[recordID]);
 
   records[recordID].used = 1;
@@ -98,21 +97,16 @@ void cdiInitRecord(stream_t *streamptr)
 {
   streamptr->record = (Record *) malloc(sizeof(Record));
 
-  streamptr->record->used       = 0;
-  streamptr->record->nrec       = 0;
-  streamptr->record->dataread   = 1;
   streamptr->record->param      = 0;
   streamptr->record->level      = 0;
   streamptr->record->date       = 0;
   streamptr->record->time       = 0;
   streamptr->record->gridID     = 0;
-  streamptr->record->zaxisID    = 0;
   streamptr->record->buffer     = NULL;
   streamptr->record->buffersize = 0;
   streamptr->record->position   = 0;
   streamptr->record->varID      = 0;
   streamptr->record->levelID    = CDI_UNDEFID;
-  streamptr->record->recid      = 0;
 }
 
 
@@ -193,7 +187,6 @@ void streamDefRecord(int streamID, int varID, int levelID)
   streamptr->record->date     = streamptr->tsteps[tsID].taxis.vdate;
   streamptr->record->time     = streamptr->tsteps[tsID].taxis.vtime;
   streamptr->record->gridID   = gridID;
-  streamptr->record->zaxisID  = zaxisID;
   streamptr->record->prec     = vlistInqVarDatatype(vlistID, varID);
 
   switch (streamptr->filetype)
@@ -445,9 +438,12 @@ void cdi_create_records(stream_t *streamptr, int tsID)
   unsigned nrecords, maxrecords;
   record_t *records;
 
-  if ( streamptr->tsteps[tsID].records ) return;
+  tsteps_t* sourceTstep = streamptr->tsteps;
+  tsteps_t* destTstep = sourceTstep + tsID;
 
-  int vlistID  = streamptr->vlistID;
+  if ( destTstep->records ) return;
+
+  int vlistID = streamptr->vlistID;
 
   if ( tsID == 0 )
     {
@@ -457,7 +453,9 @@ void cdi_create_records(stream_t *streamptr, int tsID)
 	maxrecords += (unsigned)streamptr->vars[varID].nlevs;
     }
   else
-    maxrecords = (unsigned)streamptr->tsteps[0].recordSize;
+    {
+      maxrecords = (unsigned)sourceTstep->recordSize;
+    }
 
   if ( tsID == 0 )
     {
@@ -466,48 +464,52 @@ void cdi_create_records(stream_t *streamptr, int tsID)
   else if ( tsID == 1 )
     {
       nrecords = 0;
-      maxrecords = (unsigned)streamptr->tsteps[0].recordSize;
-      for (unsigned recID = 0; recID < maxrecords; recID++ )
+      maxrecords = (unsigned)sourceTstep->recordSize;
+      for ( unsigned recID = 0; recID < maxrecords; recID++ )
 	{
-	  int varID = streamptr->tsteps[0].records[recID].varID;
-	  nrecords +=
-            (varID == -1 /* varID = -1 for write mode !!! */
-             || vlistInqVarTsteptype(vlistID, varID) != TSTEP_CONSTANT);
+	  int varID = sourceTstep->records[recID].varID;
+	  nrecords += (varID == CDI_UNDEFID /* varID = CDI_UNDEFID for write mode !!! */
+                       || vlistInqVarTsteptype(vlistID, varID) != TSTEP_CONSTANT);
+          //    printf("varID nrecords %d %d %d \n", varID, nrecords, vlistInqVarTsteptype(vlistID, varID));
 	}
     }
   else
-    nrecords = (unsigned)streamptr->tsteps[1].nallrecs;
+    {
+      nrecords = (unsigned)streamptr->tsteps[1].nallrecs;
+    }
+  //  printf("tsID, nrecords %d %d\n", tsID, nrecords);
 
   if ( maxrecords > 0 )
     records = (record_t *) malloc(maxrecords * sizeof(record_t));
   else
     records = NULL;
 
-  streamptr->tsteps[tsID].records    = records;
-  streamptr->tsteps[tsID].recordSize = (int)maxrecords;
-  streamptr->tsteps[tsID].nallrecs   = (int)nrecords;
+  destTstep->records    = records;
+  destTstep->recordSize = (int)maxrecords;
+  destTstep->nallrecs   = (int)nrecords;
 
   if ( tsID == 0 )
     {
       for ( unsigned recID = 0; recID < maxrecords; recID++ )
-	recordInitEntry(&streamptr->tsteps[tsID].records[recID]);
+        recordInitEntry(&destTstep->records[recID]);
     }
   else
     {
-      memcpy(streamptr->tsteps[tsID].records,
-	     streamptr->tsteps[0].records,
-	     (size_t)maxrecords * sizeof(record_t));
+      memcpy(destTstep->records, sourceTstep->records, (size_t)maxrecords*sizeof(record_t));
 
       for ( unsigned recID = 0; recID < maxrecords; recID++ )
 	{
-	  int varID = streamptr->tsteps[0].records[recID].varID;
-	  if ( varID != -1 ) /* varID = -1 for write mode !!! */
-	    if ( vlistInqVarTsteptype(vlistID, varID) != TSTEP_CONSTANT )
-	      {
-		streamptr->tsteps[tsID].records[recID].position = CDI_UNDEFID;
-		streamptr->tsteps[tsID].records[recID].size     = 0;
-		streamptr->tsteps[tsID].records[recID].used     = FALSE;
-	      }
+          record_t* curRecord = &sourceTstep->records[recID];
+          destTstep->records[recID].used = curRecord->used;
+          if ( curRecord->used != CDI_UNDEFID && curRecord->varID != -1 ) /* curRecord->varID = -1 for write mode !!! */
+            {
+              if ( vlistInqVarTsteptype(vlistID, curRecord->varID) != TSTEP_CONSTANT )
+                {
+                  destTstep->records[recID].position = CDI_UNDEFID;
+                  destTstep->records[recID].size     = 0;
+                  destTstep->records[recID].used     = FALSE;
+                }
+            }
 	}
     }
 }
