@@ -6,6 +6,7 @@
 #endif
 
 #include <inttypes.h>
+#include <math.h>
 #include <time.h>
 
 void
@@ -19,6 +20,15 @@ sign_flat(double v)
   return v;
 }
 
+/* data generator function */
+static inline double
+dg_wobble(double frac_x, double frac_y, double mscale, double mrscale)
+{
+  double r = sign_flat(round((cos(2.0 * M_PI * frac_x)
+                              * sin(2.0 * M_PI * frac_y)) * mscale)) * mrscale;
+  return r;
+}
+
 time_t
 cditime2time_t(int date, int timeofday);
 void
