@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -22,9 +22,6 @@
    Ensstat3       ensroccurve      Ensamble Receiver Operating Characteristics
 */
 
-#if defined(_OPENMP)
-#  include <omp.h>
-#endif
 
 #include <cdi.h>
 #include "cdo.h"
@@ -68,7 +65,6 @@ void *Ensstat3(void *argument)
   int vlistID, vlistID1, vlistID2;
   int taxisID1, taxisID2;
   int zaxisID2;
-  int ompthID;
   int *varID2;
   int time_mode;
   int **array2 = NULL;
@@ -105,7 +101,7 @@ void *Ensstat3(void *argument)
 
   if ( operfunc == func_roc ) {
     operatorInputArg("Number of eigen functions to write out");
-    nbins       = atoi(operatorArgv()[0]);
+    nbins       = parameter2int(operatorArgv()[0]);
   }
   
   nfiles = cdoStreamCnt() - 1;
@@ -121,25 +117,20 @@ void *Ensstat3(void *argument)
       if ( !userFileOverwrite(ofilename) )
 	cdoAbort("Outputfile %s already exists!", ofilename);
 
-  ef = malloc(nfiles*sizeof(ens_file_t));
+  ef = (ens_file_t*) malloc(nfiles*sizeof(ens_file_t));
 
   /* *************************************************** */
   /* should each thread be allocating memory locally???? */
   /* ("first touch strategy")                            */
   /* --> #pragma omp parallel for ...                    */
   /* *************************************************** */
-#if defined(_OPENMP)
-  field = malloc(omp_get_max_threads()*sizeof(field_t));
-  for ( i = 0; i < omp_get_max_threads(); i++ )
-#else
-  field = malloc(1*sizeof(field_t));
-  for ( i = 0; i < 1; i++ )
-#endif
+  field = (field_t*) malloc(ompNumThreads*sizeof(field_t));
+  for ( i = 0; i < ompNumThreads; i++ )
     {
       field_init(&field[i]);
       field[i].size   = nfiles;
-      field[i].ptr    = malloc(nfiles*sizeof(double));
-      field[i].weight = malloc(nfiles*sizeof(double));
+      field[i].ptr    = (double*) malloc(nfiles*sizeof(double));
+      field[i].weight = (double*) malloc(nfiles*sizeof(double));
       for ( fileID = 0; fileID < nfiles; fileID++ )
 	field[i].weight[fileID] = 1;
     }
@@ -161,9 +152,9 @@ void *Ensstat3(void *argument)
   vlistID1 = ef[0].vlistID;
   vlistID2 = vlistCreate();
   nvars = vlistNvars(vlistID1);
-  varID2 = malloc(nvars*sizeof(int));
+  varID2 = (int*) malloc(nvars*sizeof(int));
 
-  levs = calloc (nfiles, sizeof(double) );
+  levs = (double*) calloc(nfiles, sizeof(double));
   zaxisID2 = zaxisCreate(ZAXIS_GENERIC, nfiles);
   for ( i=0; i<nfiles; i++ )
     levs[i] = i;
@@ -217,37 +208,37 @@ void *Ensstat3(void *argument)
   gridsize = vlistGridsizeMax(vlistID1);
 
   for ( fileID = 0; fileID < nfiles; fileID++ )
-    ef[fileID].array = malloc(gridsize*sizeof(double));
+    ef[fileID].array = (double*) malloc(gridsize*sizeof(double));
 
   if ( operfunc == func_rank && datafunc == SPACE ) 
     { /*need to memorize data for entire grid before writing          */
-      array2 = malloc((nfiles+1)*sizeof(int*));
+      array2 = (int**) malloc((nfiles+1)*sizeof(int*));
       for ( binID=0; binID<nfiles; binID++ ) 
-	array2[binID] = calloc ( gridsize, sizeof(int) );
+	array2[binID] = (int*) calloc( gridsize, sizeof(int));
     }
   else if ( operfunc == func_rank )
     {  /* can process data separately for each timestep and only need */
        /* to cumulate values over the grid                            */
-      array2    = malloc ( (nfiles+1)*sizeof(int *));
+      array2    = (int **) malloc( (nfiles+1)*sizeof(int *));
       for ( binID=0; binID<nfiles; binID++ )
-	array2[binID] = calloc ( 1, sizeof(int) );
+	array2[binID] = (int*) calloc( 1, sizeof(int));
     }
 
   if ( operfunc == func_roc ) {
-    ctg_tab = malloc ((nbins+1)*sizeof(int*) );
-    hist =    malloc ( nbins*sizeof(int) );
-    uThresh = malloc ( nbins*sizeof(double) );
-    lThresh = malloc ( nbins*sizeof(double) );
-    roc     = malloc ((nbins+1)*sizeof(double*) );
+    ctg_tab = (int**) malloc((nbins+1)*sizeof(int*));
+    hist =    (int*) malloc ( nbins*sizeof(int));
+    uThresh = (double*) malloc( nbins*sizeof(double));
+    lThresh = (double*) malloc( nbins*sizeof(double));
+    roc     = (double**) malloc((nbins+1)*sizeof(double*));
     
     for  ( i=0; i<nbins; i++ ) {
-      ctg_tab[i] = calloc ( 4,sizeof(int) );
-      roc[i]     = calloc( 2,sizeof(double));
+      ctg_tab[i] = (int*) calloc( 4,sizeof(int));
+      roc[i]     = (double*) calloc( 2,sizeof(double));
       uThresh[i] = ((double)i+1)/nbins;
       lThresh[i] = (double)i/nbins;
     }
-    ctg_tab[nbins] = calloc (4,sizeof(int));
-    roc[nbins]     = calloc (2,sizeof(double));
+    ctg_tab[nbins] = (int*) calloc(4,sizeof(int));
+    roc[nbins]     = (double*) calloc(2,sizeof(double));
   }
   
   
@@ -300,15 +291,12 @@ void *Ensstat3(void *argument)
 	      array2[binID][0] = 0;
 
 #if defined(_OPENMP)
-#pragma omp parallel for default(shared) private(i, binID, ompthID, fileID)
+#pragma omp parallel for default(shared) private(i, binID, fileID)
 #endif
 	  for ( i = 0; i < gridsize; i++ )
 	    {
-#if defined(_OPENMP)
-	      ompthID = omp_get_thread_num();
-#else
-	      ompthID = 0;
-#endif
+	      int ompthID = cdo_omp_get_thread_num();
+
 	      field[ompthID].missval = missval;
 	      field[ompthID].nmiss = 0;
 	      have_miss = 0;
@@ -451,7 +439,7 @@ void *Ensstat3(void *argument)
       int osize = gridsize;
 
       if ( datafunc == TIME ) osize = 1;
-      tmpdoub = malloc(osize*sizeof(double));
+      tmpdoub = (double*) malloc(osize*sizeof(double));
 
       for ( binID = 0; binID < nfiles; binID++ )
 	{
