@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -91,11 +91,8 @@ static
 int getDate(const char *name)
 {
   int date = 0;
-  size_t len;
   char *pname;
 
-  len = strlen(name);
-
   pname = strchr(name, '_');
 
   if ( pname ) date = atoi(pname+1);
@@ -147,8 +144,8 @@ void *Importobs(void *argument)
 
   // printf("gridsize=%d, xsize=%d, ysize=%d\n", gridsize, xsize, ysize);
 
-  xvals = malloc(gridsize*sizeof(double));
-  yvals = malloc(gridsize*sizeof(double));
+  xvals = (double*) malloc(gridsize*sizeof(double));
+  yvals = (double*) malloc(gridsize*sizeof(double));
 
   gridInqXvals(gridID, xvals);
   gridInqYvals(gridID, yvals);
@@ -178,7 +175,7 @@ void *Importobs(void *argument)
   vlistDefTaxis(vlistID, taxisID);
 
     {
-      for ( i = 0; i < nvars; ++i ) data[i] = malloc(gridsize*sizeof(double));
+      for ( i = 0; i < nvars; ++i ) data[i] = (double*) malloc(gridsize*sizeof(double));
 
       init_vars(vlistID, gridID, zaxisID, nvars);
 
@@ -237,7 +234,7 @@ void *Importobs(void *argument)
 	  if ( i < xsize && j < ysize && index >= 0 )
 	    {
 	      pstation = station;
-	      while (isalpha(*pstation)) *pstation++;
+	      while (isalpha(*pstation)) pstation++;
 	      // printf("station %s %d\n", pstation, atoi(pstation));
 	      data[index][j*xsize+i] = value;
 	      data[    4][j*xsize+i] = height1;
