@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -55,8 +55,8 @@ void invertLevDes(int vlistID)
       /* if ( zaxisInqLevels(zaxisID1, NULL) ) */
 	{
 
-	  yv1 = malloc(nlev*sizeof(double));
-	  yv2 = malloc(nlev*sizeof(double));
+	  yv1 = (double*) malloc(nlev*sizeof(double));
+	  yv2 = (double*) malloc(nlev*sizeof(double));
 
 	  zaxisInqLevels(zaxisID1, yv1);
 	  for ( ilev = 0; ilev < nlev; ++ilev ) yv2[nlev-ilev-1] = yv1[ilev];
@@ -68,8 +68,8 @@ void invertLevDes(int vlistID)
 
       if ( zaxisInqLbounds(zaxisID1, NULL) && zaxisInqUbounds(zaxisID1, NULL) )
 	{
-	  yb1 = malloc(nlev*sizeof(double));
-	  yb2 = malloc(nlev*sizeof(double));
+	  yb1 = (double*) malloc(nlev*sizeof(double));
+	  yb2 = (double*) malloc(nlev*sizeof(double));
 
 	  zaxisInqLbounds(zaxisID1, yb1);
 	  for ( ilev = 0; ilev < nlev; ++ilev ) yb2[nlev-ilev-1] = yb1[ilev];
@@ -90,59 +90,46 @@ void invertLevDes(int vlistID)
 
 void *Invertlev(void *argument)
 {
-  int INVERTLEV;
-  int operatorID;
-  int operfunc;
-  int streamID1, streamID2;
-  int nrecs, nvars;
-  int tsID, recID, varID, levelID;
-  int gridsize;
-  int vlistID1, vlistID2;
+  int nrecs;
+  int recID, varID, levelID;
   int nmiss;
-  int **varnmiss;
-  double *array;
-  double **vardata;
-  int taxisID1, taxisID2;
-  int lcopy = FALSE;
   int nlev, nlevel;
   int gridID, zaxisID, zaxistype, offset;
+  int lcopy = FALSE;
   int linvert = FALSE;
 
   cdoInitialize(argument);
 
   if ( UNCHANGED_RECORD ) lcopy = TRUE;
 
-  INVERTLEV     = cdoOperatorAdd("invertlev",     func_all, 0, NULL);
+  cdoOperatorAdd("invertlev",     func_all, 0, NULL);
 
-  operatorID = cdoOperatorID();
-  operfunc   = cdoOperatorF1(operatorID);
+  int operatorID = cdoOperatorID();
+  int operfunc   = cdoOperatorF1(operatorID);
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = vlistDuplicate(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = vlistDuplicate(vlistID1);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  if ( operfunc == func_all || operfunc == func_hrd )
-    {
-      invertLevDes(vlistID2);
-    }
+  if ( operfunc == func_all || operfunc == func_hrd ) invertLevDes(vlistID2);
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
-  gridsize = vlistGridsizeMax(vlistID1);
+  int gridsize = vlistGridsizeMax(vlistID1);
 
-  array = malloc(gridsize*sizeof(double));
+  double *array = (double*) malloc(gridsize*sizeof(double));
 
-  nvars = vlistNvars(vlistID1);
+  int nvars = vlistNvars(vlistID1);
 
-  vardata  = malloc(nvars*sizeof(double*));
-  varnmiss = malloc(nvars*sizeof(int*));
+  double **vardata  = (double**) malloc(nvars*sizeof(double*));
+  int **varnmiss = (int**) malloc(nvars*sizeof(int*));
 
   for ( varID = 0; varID < nvars; varID++ )
     {
@@ -160,14 +147,14 @@ void *Invertlev(void *argument)
       else
 	{
 	  linvert = TRUE;
-	  vardata[varID]  = malloc(gridsize*nlev*sizeof(double));
-	  varnmiss[varID] = malloc(nlev*sizeof(int));
+	  vardata[varID]  = (double*) malloc(gridsize*nlev*sizeof(double));
+	  varnmiss[varID] = (int*) malloc(nlev*sizeof(int));
 	}
     }
 
   if ( linvert == FALSE ) cdoWarning("No variables with invertable levels found!");
 
-  tsID = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
       taxisCopyTimestep(taxisID2, taxisID1);
