@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -35,150 +35,27 @@
 #include "pstream.h"
 
 
-void datetime_avg_dtinfo(int calendar, int ndates, dtinfo_t *dtinfo)
-{
-  int vdate, vtime;
-  juldate_t juldate1, juldate2, juldatem;
-  double seconds;
-  /*
-  for ( i = 0; i < ndates; i++ )
-    fprintf(stdout, "%4d %d %d\n", i+1, dtinfo[i].v.date, dtinfo[i].v.time);
-  */
-  if ( ndates%2 == 0 )
-    {
-      /*
-      vdate = dtinfo[ndates-1].v.date;
-      vtime = dtinfo[ndates-1].v.time;
-      */
-      vdate = dtinfo[ndates/2-1].v.date;
-      vtime = dtinfo[ndates/2-1].v.time;
-      juldate1 = juldate_encode(calendar, vdate, vtime);
-
-      vdate = dtinfo[ndates/2].v.date;
-      vtime = dtinfo[ndates/2].v.time;
-      juldate2 = juldate_encode(calendar, vdate, vtime);
-
-      seconds = juldate_to_seconds(juldate_sub(juldate2, juldate1)) / 2;
-      juldatem = juldate_add_seconds(NINT(seconds), juldate1);
-      juldate_decode(calendar, juldatem, &vdate, &vtime);
-    }
-  else
-    {
-      vdate = dtinfo[ndates/2].v.date;
-      vtime = dtinfo[ndates/2].v.time;
-    }
-
-  dtinfo[ndates].v.date = vdate;
-  dtinfo[ndates].v.time = vtime;
-  /*
-  fprintf(stdout, "res: %d %d\n\n", dtinfo[ndates].v.date, dtinfo[ndates].v.time);
-  */
-}
-
-
-void datetime_avg(int calendar, int ndates, datetime_t *datetime)
-{
-  int vdate, vtime;
-  juldate_t juldate1, juldate2, juldatem;
-  double seconds;
-  /*
-  for ( i = 0; i < ndates; i++ )
-    fprintf(stdout, "%4d %d %d\n", i+1, datetime[i].date, datetime[i].time);
-  */
-  if ( ndates%2 == 0 )
-    {
-      /*
-      vdate = datetime[ndates-1].date;
-      vtime = datetime[ndates-1].time;
-      */
-      vdate = datetime[ndates/2-1].date;
-      vtime = datetime[ndates/2-1].time;
-      juldate1 = juldate_encode(calendar, vdate, vtime);
-
-      vdate = datetime[ndates/2].date;
-      vtime = datetime[ndates/2].time;
-      juldate2 = juldate_encode(calendar, vdate, vtime);
-
-      seconds = juldate_to_seconds(juldate_sub(juldate2, juldate1)) / 2;
-      juldatem = juldate_add_seconds(NINT(seconds), juldate1);
-      juldate_decode(calendar, juldatem, &vdate, &vtime);
-    }
-  else
-    {
-      vdate = datetime[ndates/2].date;
-      vtime = datetime[ndates/2].time;
-    }
-
-  datetime[ndates].date = vdate;
-  datetime[ndates].time = vtime;
-  /*
-  fprintf(stdout, "res: %d %d\n\n", datetime[ndates].date, datetime[ndates].time);
-  */
-}
-
-
-void get_timestat_date(int *tstat_date)
-{
-  char *envstr;
-
-  envstr = getenv("TIMESTAT_DATE");
-  if ( envstr == NULL ) envstr = getenv("RUNSTAT_DATE");
-  if ( envstr )
-    {
-      int env_date = -1;
-      char envstrl[8];
-
-      memcpy(envstrl, envstr, 8);
-      envstrl[7] = 0;
-      strtolower(envstrl);
-
-      if      ( memcmp(envstrl, "first", 5)  == 0 )  env_date = DATE_FIRST;
-      else if ( memcmp(envstrl, "last", 4)   == 0 )  env_date = DATE_LAST;
-      else if ( memcmp(envstrl, "middle", 6) == 0 )  env_date = DATE_MIDDLE;
-
-      if ( env_date >= 0 )
-	{
-	  *tstat_date = env_date;
-
-	  if ( cdoVerbose ) cdoPrint("Set TIMESTAT_DATE to %s", envstr);
-	}
-    }
-}
-
-
 void *Runstat(void *argument)
 {
-  int operatorID;
-  int operfunc;
-  int gridsize, gridsizemax;
+  int timestat_date = TIMESTAT_MEAN;
+  int gridsize;
   int i;
   int varID;
   int recID;
-  int nrecs, nrecords;
+  int nrecs;
   int levelID;
   int tsID;
   int otsID;
-  int inp, its, ndates = 0;
-  int streamID1, streamID2;
-  int vlistID1, vlistID2;
+  int inp, its;
   int nmiss;
-  int nvars, nlevel;
-  int *recVarID, *recLevelID;
-  int lmean = FALSE, lvarstd = FALSE, lstd = FALSE;
-  int *imask;
-  double missval;
-  double divisor;
-  field_t ***vars1 = NULL, ***vars2 = NULL, ***samp1 = NULL;
-  dtinfo_t *dtinfo;
-  int taxisID1, taxisID2;
-  int calendar;
+  int nlevel;
   int runstat_nomiss = 0;
-  int timestat_date = DATE_MIDDLE;
-  char *envstr;
+  double missval;
+  field_t ***vars1 = NULL, ***vars2 = NULL, ***samp1 = NULL;
 
   cdoInitialize(argument);
 
-  envstr = getenv("RUNSTAT_NOMISS");
+  char *envstr = getenv("RUNSTAT_NOMISS");
   if ( envstr )
     {
       char *endptr;
@@ -186,8 +63,6 @@ void *Runstat(void *argument)
       if ( envval == 1 ) runstat_nomiss = 1;
     }
 
-  get_timestat_date(&timestat_date);
-
   cdoOperatorAdd("runmin",  func_min,  0, NULL);
   cdoOperatorAdd("runmax",  func_max,  0, NULL);
   cdoOperatorAdd("runsum",  func_sum,  0, NULL);
@@ -198,44 +73,45 @@ void *Runstat(void *argument)
   cdoOperatorAdd("runstd",  func_std,  0, NULL);
   cdoOperatorAdd("runstd1", func_std1, 0, NULL);
 
-  operatorID = cdoOperatorID();
-  operfunc = cdoOperatorF1(operatorID);
+  int operatorID = cdoOperatorID();
+  int operfunc = cdoOperatorF1(operatorID);
 
   operatorInputArg("number of timesteps");
-  ndates = atoi(operatorArgv()[0]);
+  int ndates = parameter2int(operatorArgv()[0]);
 
-  lmean   = operfunc == func_mean || operfunc == func_avg;
-  lstd    = operfunc == func_std || operfunc == func_std1;
-  lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
-  divisor = operfunc == func_std1 || operfunc == func_var1;
+  int lmean   = operfunc == func_mean || operfunc == func_avg;
+  int lstd    = operfunc == func_std || operfunc == func_std1;
+  int lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
+  double divisor = operfunc == func_std1 || operfunc == func_var1;
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = vlistDuplicate(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = vlistDuplicate(vlistID1);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  calendar = taxisInqCalendar(taxisID1);
-
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
-  nvars    = vlistNvars(vlistID1);
-  nrecords = vlistNrecs(vlistID1);
+  int nvars    = vlistNvars(vlistID1);
+  int nrecords = vlistNrecs(vlistID1);
 
-  recVarID   = malloc(nrecords*sizeof(int));
-  recLevelID = malloc(nrecords*sizeof(int));
+  int *recVarID   = (int*) malloc(nrecords*sizeof(int));
+  int *recLevelID = (int*) malloc(nrecords*sizeof(int));
 
-  dtinfo = malloc((ndates+1)*sizeof(dtinfo_t));
-  vars1 = malloc((ndates+1)*sizeof(field_t **));
+  dtlist_type *dtlist = dtlist_new();
+  dtlist_set_stat(dtlist, timestat_date);
+  dtlist_set_calendar(dtlist, taxisInqCalendar(taxisID1));
+
+  vars1 = (field_t ***) malloc((ndates+1)*sizeof(field_t **));
   if ( !runstat_nomiss )
-    samp1 = malloc((ndates+1)*sizeof(field_t **));
+    samp1 = (field_t ***) malloc((ndates+1)*sizeof(field_t **));
   if ( lvarstd )
-    vars2 = malloc((ndates+1)*sizeof(field_t **));
+    vars2 = (field_t ***) malloc((ndates+1)*sizeof(field_t **));
 
   for ( its = 0; its < ndates; its++ )
     {
@@ -246,15 +122,15 @@ void *Runstat(void *argument)
 	vars2[its] = field_malloc(vlistID1, FIELD_PTR);
     }
 
-  gridsizemax = vlistGridsizeMax(vlistID1);
-  imask = malloc(gridsizemax*sizeof(int));
+  int gridsizemax = vlistGridsizeMax(vlistID1);
+  int *imask = (int*) malloc(gridsizemax*sizeof(int));
 
   for ( tsID = 0; tsID < ndates; tsID++ )
     {
       nrecs = streamInqTimestep(streamID1, tsID);
       if ( nrecs == 0 ) cdoAbort("File has less then %d timesteps!", ndates);
 
-      taxisInqDTinfo(taxisID1, &dtinfo[tsID]);
+      dtlist_taxisInqTimestep(dtlist, taxisID1, tsID);
 	
       for ( recID = 0; recID < nrecs; recID++ )
 	{
@@ -361,17 +237,7 @@ void *Runstat(void *argument)
 	      }
 	  }
 
-      if      ( timestat_date == DATE_MIDDLE ) datetime_avg_dtinfo(calendar, ndates, dtinfo);
-      else if ( timestat_date == DATE_FIRST  ) dtinfo[ndates].v = dtinfo[0].v;
-      else if ( timestat_date == DATE_LAST   ) dtinfo[ndates].v = dtinfo[ndates-1].v;
-
-      if ( taxisHasBounds(taxisID2) )
-	{
-	  dtinfo[ndates].b[0] = dtinfo[0].b[0];
-	  dtinfo[ndates].b[1] = dtinfo[ndates-1].b[1];
-	}
-
-      taxisDefDTinfo(taxisID2, dtinfo[ndates]);
+      dtlist_stat_taxisDefTimestep(dtlist, taxisID2, ndates);
       streamDefTimestep(streamID2, otsID);
 
       for ( recID = 0; recID < nrecords; recID++ )
@@ -387,7 +253,8 @@ void *Runstat(void *argument)
 
       otsID++;
 
-      dtinfo[ndates] = dtinfo[0];
+      dtlist_shift(dtlist);
+
       vars1[ndates] = vars1[0];
       if ( !runstat_nomiss )
 	samp1[ndates] = samp1[0];
@@ -396,7 +263,6 @@ void *Runstat(void *argument)
 
       for ( inp = 0; inp < ndates; inp++ )
 	{
-	  dtinfo[inp] = dtinfo[inp+1];
 	  vars1[inp] = vars1[inp+1];
 	  if ( !runstat_nomiss )
 	    samp1[inp] = samp1[inp+1];
@@ -407,7 +273,7 @@ void *Runstat(void *argument)
       nrecs = streamInqTimestep(streamID1, tsID);
       if ( nrecs == 0 ) break;
 
-      taxisInqDTinfo(taxisID1, &dtinfo[ndates-1]);
+      dtlist_taxisInqTimestep(dtlist, taxisID1, ndates-1);
 
       for ( recID = 0; recID < nrecs; recID++ )
 	{
@@ -477,7 +343,6 @@ void *Runstat(void *argument)
       if ( lvarstd ) field_free(vars2[its], vlistID1);
     }
 
-  free(dtinfo);
   free(vars1);
   if ( !runstat_nomiss ) free(samp1);
   if ( lvarstd ) free(vars2);
@@ -486,6 +351,8 @@ void *Runstat(void *argument)
   if ( recLevelID ) free(recLevelID);
   if ( imask )      free(imask);
 
+  dtlist_delete(dtlist);
+
   streamClose(streamID2);
   streamClose(streamID1);
 
