@@ -32,27 +32,22 @@
 
 void *Seaspctl(void *argument)
 {
-  int gridsize;
-  int vdate1 = 0, vtime1 = 0;
+  int timestat_date = TIMESTAT_MEAN;
+  int vdate1 = 0;
   int vdate2 = 0, vtime2 = 0;
   int vdate3 = 0, vtime3 = 0;
-  int vdate4 = 0, vtime4 = 0;
-  int nrecs, nrecords;
+  int nrecs;
   int gridID, varID, levelID, recID;
   int tsID;
   int otsID;
   long nsets;
   int year, month, day, seas, seas0 = 0;
-  int streamID1, streamID2, streamID3, streamID4;
-  int vlistID1, vlistID2, vlistID3, vlistID4, taxisID1, taxisID2, taxisID3, taxisID4;
   int nmiss;
-  int nvars, nlevels;
-  int *recVarID, *recLevelID;
+  int nlevels;
   int newseas, oldmon = 0, newmon;
   double missval;
   field_t **vars1 = NULL;
   field_t field;
-  double pn;
   HISTOGRAM_SET *hset = NULL;
   int season_start;
 
@@ -61,48 +56,52 @@ void *Seaspctl(void *argument)
   cdoOperatorAdd("seaspctl", func_pctl, 0, NULL);
 
   operatorInputArg("percentile number");
-  pn = atof(operatorArgv()[0]);
+  double pn = parameter2double(operatorArgv()[0]);
       
   if ( !(pn > 0 && pn < 100) )
     cdoAbort("Illegal argument: percentile number %g is not in the range 0..100!", pn);
 
   season_start = get_season_start();
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
-  streamID2 = streamOpenRead(cdoStreamName(1));
-  streamID3 = streamOpenRead(cdoStreamName(2));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID2 = streamOpenRead(cdoStreamName(1));
+  int streamID3 = streamOpenRead(cdoStreamName(2));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = streamInqVlist(streamID2);
-  vlistID3 = streamInqVlist(streamID3);
-  vlistID4 = vlistDuplicate(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = streamInqVlist(streamID2);
+  int vlistID3 = streamInqVlist(streamID3);
+  int vlistID4 = vlistDuplicate(vlistID1);
 
   vlistCompare(vlistID1, vlistID2, CMP_ALL);
   vlistCompare(vlistID1, vlistID3, CMP_ALL);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = vlistInqTaxis(vlistID2);
-  taxisID3 = vlistInqTaxis(vlistID3);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = vlistInqTaxis(vlistID2);
+  int taxisID3 = vlistInqTaxis(vlistID3);
   /* TODO - check that time axes 2 and 3 are equal */
 
-  taxisID4 = taxisDuplicate(taxisID1);
+  int taxisID4 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID4, taxisID4);
 
-  streamID4 = streamOpenWrite(cdoStreamName(3), cdoFiletype());
+  int streamID4 = streamOpenWrite(cdoStreamName(3), cdoFiletype());
 
   streamDefVlist(streamID4, vlistID4);
 
-  nvars    = vlistNvars(vlistID1);
-  nrecords = vlistNrecs(vlistID1);
+  int nvars    = vlistNvars(vlistID1);
+  int nrecords = vlistNrecs(vlistID1);
 
-  recVarID   = malloc(nrecords * sizeof(int));
-  recLevelID = malloc(nrecords * sizeof(int));
+  int *recVarID   = (int*) malloc(nrecords * sizeof(int));
+  int *recLevelID = (int*) malloc(nrecords * sizeof(int));
 
-  gridsize = vlistGridsizeMax(vlistID1);
+  dtlist_type *dtlist = dtlist_new();
+  dtlist_set_stat(dtlist, timestat_date);
+  dtlist_set_calendar(dtlist, taxisInqCalendar(taxisID1));
 
-  field.ptr = malloc(gridsize*sizeof(double));
+  int gridsize = vlistGridsizeMax(vlistID1);
 
-  vars1 = malloc(nvars * sizeof(field_t *));
+  field.ptr = (double*) malloc(gridsize*sizeof(double));
+
+  vars1 = (field_t **) malloc(nvars * sizeof(field_t *));
   hset = hsetCreate(nvars);
 
   for ( varID = 0; varID < nvars; varID++ )
@@ -112,7 +111,7 @@ void *Seaspctl(void *argument)
       nlevels   = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
       missval  = vlistInqVarMissval(vlistID1, varID);
 
-      vars1[varID] = malloc(nlevels * sizeof(field_t));
+      vars1[varID] = (field_t*) malloc(nlevels * sizeof(field_t));
       hsetCreateVarLevels(hset, varID, nlevels, gridID);
 
       for ( levelID = 0; levelID < nlevels; levelID++ )
@@ -120,7 +119,7 @@ void *Seaspctl(void *argument)
 	  vars1[varID][levelID].grid    = gridID;
 	  vars1[varID][levelID].nmiss   = 0;
 	  vars1[varID][levelID].missval = missval;
-	  vars1[varID][levelID].ptr     = malloc(gridsize * sizeof(double));
+	  vars1[varID][levelID].ptr     = (double*) malloc(gridsize * sizeof(double));
 	}
     }
 
@@ -128,9 +127,6 @@ void *Seaspctl(void *argument)
   otsID   = 0;
   while ( TRUE )
     {
-      nsets   = 0;
-      newseas = FALSE;
-
       nrecs = streamInqTimestep(streamID2, otsID);
       if ( nrecs != streamInqTimestep(streamID3, otsID) )
         cdoAbort("Number of records at time step %d of %s and %s differ!", otsID+1, cdoStreamName(1)->args, cdoStreamName(2)->args);
@@ -148,6 +144,7 @@ void *Seaspctl(void *argument)
 	  streamReadRecord(streamID2, vars1[varID][levelID].ptr, &nmiss);
           vars1[varID][levelID].nmiss = nmiss;
         }
+
       for ( recID = 0; recID < nrecs; recID++ )
         {
           streamInqRecord(streamID3, &varID, &levelID);
@@ -159,10 +156,13 @@ void *Seaspctl(void *argument)
 	  hsetDefVarLevelBounds(hset, varID, levelID, &vars1[varID][levelID], &field);
         }
 
+      nsets   = 0;
+      newseas = FALSE;
       while ( nrecs && (nrecs = streamInqTimestep(streamID1, tsID)) )
 	{
-	  vdate1 = taxisInqVdate(taxisID1);
-	  vtime1 = taxisInqVtime(taxisID1);
+	  dtlist_taxisInqTimestep(dtlist, taxisID1, nsets);
+	  vdate1 = dtlist_get_vdate(dtlist, nsets);
+
 	  cdiDecodeDate(vdate1, &year, &month, &day);
 	  if ( month < 0 || month > 16 )
 	    cdoAbort("Month %d out of range!", month);
@@ -215,19 +215,12 @@ void *Seaspctl(void *argument)
 	      hsetAddVarLevelValues(hset, varID, levelID, &vars1[varID][levelID]);
 	    }
 
-	  vdate4 = vdate1;
-	  vtime4 = vtime1;
 	  nsets++;
 	  tsID++;
 	}
 
       if ( nrecs == 0 && nsets == 0 ) break;
 
-      if ( vdate2 != vdate4 )
-        cdoAbort("Verification dates at time step %d of %s, %s and %s differ!", otsID+1, cdoStreamName(1)->args, cdoStreamName(2)->args, cdoStreamName(3)->args);
-      if ( vtime2 != vtime4 )
-        cdoAbort("Verification times at time step %d of %s, %s and %s differ!", otsID+1, cdoStreamName(1)->args, cdoStreamName(2)->args, cdoStreamName(3)->args);
-
       for ( varID = 0; varID < nvars; varID++ )
 	{
 	  if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT ) continue;
@@ -237,8 +230,7 @@ void *Seaspctl(void *argument)
             hsetGetVarLevelPercentiles(&vars1[varID][levelID], hset, varID, levelID, pn);
 	}
 
-      taxisDefVdate(taxisID4, vdate4);
-      taxisDefVtime(taxisID4, vtime4);
+      dtlist_stat_taxisDefTimestep(dtlist, taxisID4, nsets);
       streamDefTimestep(streamID4, otsID);
 
       for ( recID = 0; recID < nrecords; recID++ )
@@ -267,6 +259,8 @@ void *Seaspctl(void *argument)
   free(vars1);
   hsetDestroy(hset);
 
+  dtlist_delete(dtlist);
+
   if ( field.ptr ) free(field.ptr);
 
   if ( recVarID   ) free(recVarID);
