@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -46,12 +46,7 @@
 
 void *Selvar(void *argument)
 {
-  int SELPARAM, SELCODE, SELNAME, SELLEVEL, SELLEVIDX, SELGRID, SELZAXIS, SELLTYPE; 
-  int SELTABNUM, DELPARAM, DELCODE, DELNAME, SELSTDNAME;
-  int operatorID;
-  int streamID1, streamID2;
-  int tsID, nrecs;
-  int nvars, nlevs;
+  int nlevs;
   int code, tabnum, param, gridID, zaxisID, levID;
   int grididx, zaxisidx;
   double level;
@@ -64,49 +59,49 @@ void *Selvar(void *argument)
   char varname[CDI_MAX_NAME];
   char stdname[CDI_MAX_NAME];
   char gridname[CDI_MAX_NAME];
+  char zaxistypename[CDI_MAX_NAME];
   char zaxisname[CDI_MAX_NAME];
   char **argnames = NULL;
-  int vlistID1 = -1, vlistID2 = -1;
   int isel;
   int i;
-  int npar;
-  int intlist, byname;
   int lcopy = FALSE;
   int gridsize;
   int nmiss;
   double *array = NULL;
-  int taxisID1, taxisID2;
-  int ltype;
   LIST *ilist = listNew(INT_LIST);
   LIST *flist = listNew(FLT_LIST);
 
   cdoInitialize(argument);
 
-  SELPARAM     = cdoOperatorAdd("selparam",     0, 0, "parameters");
-  SELCODE      = cdoOperatorAdd("selcode",      0, 0, "code numbers");
-  SELNAME      = cdoOperatorAdd("selname",      0, 0, "variable names");
-  SELSTDNAME   = cdoOperatorAdd("selstdname",   0, 0, "standard names");
-  SELLEVEL     = cdoOperatorAdd("sellevel",     0, 0, "levels");
-  SELLEVIDX    = cdoOperatorAdd("sellevidx",    0, 0, "index of levels");
-  SELGRID      = cdoOperatorAdd("selgrid",      0, 0, "list of grid names or numbers");
-  SELZAXIS     = cdoOperatorAdd("selzaxis",     0, 0, "list of zaxis names or numbers");
-  SELTABNUM    = cdoOperatorAdd("seltabnum",    0, 0, "table numbers");
-  DELPARAM     = cdoOperatorAdd("delparam",     0, 0, "parameter");
-  DELCODE      = cdoOperatorAdd("delcode",      0, 0, "code numbers");
-  DELNAME      = cdoOperatorAdd("delname",      0, 0, "variable names");
-  SELLTYPE     = cdoOperatorAdd("selltype",     0, 0, "GRIB level types"); 
+# define INVERTS_SELECTION(id) (cdoOperatorF2(id) & 1)
+# define TAKES_STRINGS(id) (cdoOperatorF2(id) & 2)
+# define TAKES_INTEGERS(id) (cdoOperatorF2(id) & 4)
+# define TAKES_FLOATS(id) (cdoOperatorF2(id) & 8)
+
+  int SELPARAM     = cdoOperatorAdd("selparam",     0, 2, "parameters");
+  int SELCODE      = cdoOperatorAdd("selcode",      0, 4, "code numbers");
+  int SELNAME      = cdoOperatorAdd("selname",      0, 2, "variable names");
+  int SELSTDNAME   = cdoOperatorAdd("selstdname",   0, 2, "standard names");
+  int SELLEVEL     = cdoOperatorAdd("sellevel",     0, 8, "levels");
+  int SELLEVIDX    = cdoOperatorAdd("sellevidx",    0, 4, "index of levels");
+  int SELGRID      = cdoOperatorAdd("selgrid",      0, 4|2, "list of grid names or numbers");
+  int SELZAXIS     = cdoOperatorAdd("selzaxis",     0, 4|2, "list of zaxis types or numbers");
+  int SELZAXISNAME = cdoOperatorAdd("selzaxisname", 0, 2, "list of zaxis names");
+  int SELTABNUM    = cdoOperatorAdd("seltabnum",    0, 4, "table numbers");
+  int DELPARAM     = cdoOperatorAdd("delparam",     0, 2|1, "parameter");
+  int DELCODE      = cdoOperatorAdd("delcode",      0, 1, "code numbers");
+  int DELNAME      = cdoOperatorAdd("delname",      0, 2|1, "variable names");
+  int SELLTYPE     = cdoOperatorAdd("selltype",     0, 4, "GRIB level types"); 
 
   if ( UNCHANGED_RECORD ) lcopy = TRUE;
 
-  operatorID = cdoOperatorID();
+  int operatorID = cdoOperatorID();
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
-  intlist = FALSE;
-  byname  = TRUE;
+  int args_are_numeric = operatorArgc() > 0 && isdigit(*operatorArgv()[0]);
 
-  if ( operatorID == SELPARAM || operatorID == DELPARAM || operatorID == SELNAME || operatorID == DELNAME || 
-       operatorID == SELSTDNAME || operatorID == SELGRID || operatorID == SELZAXIS )
+  if ( TAKES_STRINGS(operatorID) && !( TAKES_INTEGERS(operatorID) && args_are_numeric ) )
     {
       nsel     = operatorArgc();
       argnames = operatorArgv();
@@ -114,17 +109,8 @@ void *Selvar(void *argument)
       if ( cdoVerbose )
 	for ( i = 0; i < nsel; i++ )
 	  fprintf(stderr, "name %d = %s\n", i+1, argnames[i]);
-
-      if ( operatorID == SELGRID || operatorID == SELZAXIS )
-	{
-	  if ( nsel > 0 && isdigit(*argnames[0]) )
-	    {
-	      intlist = TRUE;
-	      byname  = FALSE;
-	    }
-	}
     }
-  else if ( operatorID == SELLEVEL )
+  else if ( TAKES_FLOATS(operatorID) )
     {
       nsel = args2fltlist(operatorArgc(), operatorArgv(), flist);
       fltarr = (double *) listArrayPtr(flist);
@@ -134,11 +120,6 @@ void *Selvar(void *argument)
 	  printf("flt %d = %g\n", i+1, fltarr[i]);
     }
   else
-    {
-      intlist = TRUE;
-    }
-
-  if ( intlist )
     {
       nsel = args2intlist(operatorArgc(), operatorArgv(), ilist);
       intarr = (int *) listArrayPtr(ilist);
@@ -150,7 +131,7 @@ void *Selvar(void *argument)
 
   if ( nsel )
     {
-      selfound = malloc(nsel*sizeof(int));
+      selfound = (int*) malloc(nsel*sizeof(int));
       for ( i = 0; i < nsel; i++ ) selfound[i] = FALSE;
     }
 
@@ -158,12 +139,12 @@ void *Selvar(void *argument)
   if ( nsel == 0 )
     cdoAbort("missing code argument!");
   */
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
+  int vlistID1 = streamInqVlist(streamID1);
 
   vlistClearFlag(vlistID1);
-  nvars = vlistNvars(vlistID1);
+  int nvars = vlistNvars(vlistID1);
   for ( varID = 0; varID < nvars; varID++ )
     {
       vlistInqVarName(vlistID1, varID, varname);
@@ -177,7 +158,8 @@ void *Selvar(void *argument)
       zaxisidx = vlistZaxisIndex(vlistID1, zaxisID);
       nlevs    = zaxisInqSize(zaxisID);
       gridName(gridInqType(gridID), gridname);
-      zaxisName(zaxisInqType(zaxisID), zaxisname);
+      zaxisInqName(zaxisID, zaxisname);
+      zaxisName(zaxisInqType(zaxisID), zaxistypename);
 
       cdiParamToString(param, paramstr, sizeof(paramstr));
 
@@ -190,133 +172,83 @@ void *Selvar(void *argument)
 
 	  for ( isel = 0; isel < nsel; isel++ )
 	    {
+              int found = 0;
 	      if ( operatorID == SELCODE )
 		{
-		  if ( intarr[isel] == code )
-		    {
-		      vlistDefFlag(vlistID1, varID, levID, TRUE);
-		      selfound[isel] = TRUE;
-		    }
+		  found = intarr[isel] == code;
 		}
 	      else if ( operatorID == SELPARAM )
 		{
-		  if ( strcmp(argnames[isel], paramstr) == 0 )
-		    {
-		      vlistDefFlag(vlistID1, varID, levID, TRUE);
-		      selfound[isel] = TRUE;
-		    }
+		  found = wildcardmatch(argnames[isel], paramstr) == 0;
 		}
 	      else if ( operatorID == SELNAME )
 		{
-		  if ( strcmp(argnames[isel], varname) == 0 )
-		    {
-		      vlistDefFlag(vlistID1, varID, levID, TRUE);
-		      selfound[isel] = TRUE;
-		    }
+		  found = wildcardmatch(argnames[isel], varname) == 0;
 		}
 	      else if ( operatorID == SELSTDNAME )
 		{
-		  if ( strcmp(argnames[isel], stdname) == 0 )
-		    {
-		      vlistDefFlag(vlistID1, varID, levID, TRUE);
-		      selfound[isel] = TRUE;
-		    }
+		  found = strcmp(argnames[isel], stdname) == 0;
 		}
 	      else if ( operatorID == SELLEVEL )
 		{
-		  if ( fabs(fltarr[isel] - level) < 0.0001 )
-		    {
-		      vlistDefFlag(vlistID1, varID, levID, TRUE);
-		      selfound[isel] = TRUE;
-		    }
+		  found = fabs(fltarr[isel] - level) < 0.0001;
 		}
 	      else if ( operatorID == SELLEVIDX )
 		{
-		  if ( intarr[isel] == (levID+1) )
-		    {
-		      vlistDefFlag(vlistID1, varID, levID, TRUE);
-		      selfound[isel] = TRUE;
-		    }
+		  found = intarr[isel] == (levID+1);
 		}
-	      else if ( operatorID == SELGRID && byname == FALSE )
+	      else if ( operatorID == SELGRID && args_are_numeric )
 		{
-		  if ( intarr[isel] == (grididx+1) )
-		    {
-		      vlistDefFlag(vlistID1, varID, levID, TRUE);
-		      selfound[isel] = TRUE;
-		    }
+		  found = intarr[isel] == (grididx+1);
 		}
-	      else if ( operatorID == SELGRID && byname == TRUE )
+	      else if ( operatorID == SELGRID && !args_are_numeric )
 		{
-		  if ( memcmp(argnames[isel], gridname, strlen(argnames[isel])) == 0 )
-		    {
-		      vlistDefFlag(vlistID1, varID, levID, TRUE);
-		      selfound[isel] = TRUE;
-		    }
+		  found = memcmp(argnames[isel], gridname, strlen(argnames[isel])) == 0;
 		}
-	      else if ( operatorID == SELZAXIS && byname == FALSE )
+	      else if ( operatorID == SELZAXIS && args_are_numeric )
 		{
-		  if ( intarr[isel] == (zaxisidx+1) )
-		    {
-		      vlistDefFlag(vlistID1, varID, levID, TRUE);
-		      selfound[isel] = TRUE;
-		    }
+		  found = intarr[isel] == (zaxisidx+1);
 		}
-	      else if ( operatorID == SELZAXIS && byname == TRUE )
+	      else if ( operatorID == SELZAXIS && !args_are_numeric )
 		{
-		  if ( memcmp(argnames[isel], zaxisname, strlen(argnames[isel])) == 0 )
-		    {
-		      vlistDefFlag(vlistID1, varID, levID, TRUE);
-		      selfound[isel] = TRUE;
-		    }
+		  found = memcmp(argnames[isel], zaxistypename, strlen(argnames[isel])) == 0;
+		}
+	      else if ( operatorID == SELZAXISNAME )
+		{
+		  found = wildcardmatch(argnames[isel], zaxisname) == 0;
 		}
 	      else if ( operatorID == SELTABNUM )
 		{
-		  if ( intarr[isel] == tabnum )
-		    {
-		      vlistDefFlag(vlistID1, varID, levID, TRUE);
-		      selfound[isel] = TRUE;
-		    }
+		  found = intarr[isel] == tabnum;
 		}
 	      else if ( operatorID == DELCODE )
 		{
-		  if ( intarr[isel] == code )
-		    {
-		      vlistDefFlag(vlistID1, varID, levID, FALSE);
-		      selfound[isel] = TRUE;
-		    }
+		  found = intarr[isel] == code;
 		}
 	      else if ( operatorID == DELNAME )
 		{
-		  if ( strcmp(argnames[isel], varname) == 0 )
-		    {
-		      vlistDefFlag(vlistID1, varID, levID, FALSE);
-		      selfound[isel] = TRUE;
-		    }
+		  found = wildcardmatch(argnames[isel], varname) == 0;
 		}
 	      else if ( operatorID == DELPARAM )
 		{
-		  if ( strcmp(argnames[isel], paramstr) == 0 )
-		    {
-		      vlistDefFlag(vlistID1, varID, levID, FALSE);
-		      selfound[isel] = TRUE;
-		    }
+		  found = strcmp(argnames[isel], paramstr) == 0;
 		}
 	      else if ( operatorID == SELLTYPE )
 		{
-		  ltype = zaxis2ltype(zaxisID);
-
-		  if ( intarr[isel] == ltype )
-		    {
-		      vlistDefFlag(vlistID1, varID, levID, TRUE);
-		      selfound[isel] = TRUE;
-		    }
+		  found = intarr[isel] == zaxis2ltype(zaxisID);
 		}
+
+	      if ( found )
+	        {
+		  vlistDefFlag(vlistID1, varID, levID, !INVERTS_SELECTION(operatorID));
+		  selfound[isel] = TRUE;
+	        }
+
 	    }
 	}
     }
 
-  npar = 0;
+  int npar = 0;
   for ( varID = 0; varID < nvars; varID++ )
     {
       zaxisID = vlistInqVarZaxis(vlistID1, varID);
@@ -356,19 +288,23 @@ void *Selvar(void *argument)
 	    {
 	      cdoWarning("Level index %d not found!", intarr[isel]);
 	    }
-	  else if ( operatorID == SELGRID && byname == FALSE )
+	  else if ( operatorID == SELGRID && args_are_numeric )
 	    {
 	      cdoWarning("Grid %d not found!", intarr[isel]);
 	    }
-	  else if ( operatorID == SELGRID && byname == TRUE )
+	  else if ( operatorID == SELGRID && !args_are_numeric )
 	    {
 	      cdoWarning("Grid name %s not found!", argnames[isel]);
 	    }
-	  else if ( operatorID == SELZAXIS && byname == FALSE )
+	  else if ( operatorID == SELZAXIS && args_are_numeric )
 	    {
 	      cdoWarning("Zaxis %d not found!", intarr[isel]);
 	    }
-	  else if ( operatorID == SELZAXIS && byname == TRUE )
+	  else if ( operatorID == SELZAXIS && !args_are_numeric )
+	    {
+	      cdoWarning("Zaxis type %s not found!", argnames[isel]);
+	    }
+	  else if ( operatorID == SELZAXISNAME )
 	    {
 	      cdoWarning("Zaxis name %s not found!", argnames[isel]);
 	    }
@@ -382,10 +318,10 @@ void *Selvar(void *argument)
 	    }
 	}
     }
-  
+
   if ( npar == 0 ) cdoAbort("No variables selected!");
 
-  vlistID2 = vlistCreate();
+  int vlistID2 = vlistCreate();
   vlistCopyFlag(vlistID2, vlistID1);
 
   nvars = vlistNvars(vlistID2);
@@ -393,13 +329,13 @@ void *Selvar(void *argument)
     if ( vlistInqVarTsteptype(vlistID2, varID) != TSTEP_CONSTANT ) break;
   if ( varID == nvars ) vlistDefNtsteps(vlistID2, 0);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  nrecs = vlistNrecs(vlistID2);
+  int nrecs = vlistNrecs(vlistID2);
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
@@ -407,10 +343,10 @@ void *Selvar(void *argument)
     {
       gridsize = vlistGridsizeMax(vlistID1);
       if ( vlistNumber(vlistID1) != CDI_REAL ) gridsize *= 2;
-      array = malloc(gridsize*sizeof(double));
+      array = (double*) malloc(gridsize*sizeof(double));
     }
 
-  tsID = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
       taxisCopyTimestep(taxisID2, taxisID1);
