@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -22,10 +22,6 @@
 */
 
 
-#if defined(_OPENMP)
-#  include <omp.h>
-#endif
-
 #include <cdi.h>
 #include "cdo.h"
 #include "cdo_int.h"
@@ -62,7 +58,6 @@ void *Timsort(void *argument)
   int nmiss;
   int nvars, nlevel;
   int *vdate = NULL, *vtime = NULL;
-  int ompthID;
   double **sarray = NULL;
   field_t ***vars = NULL;
 
@@ -89,9 +84,9 @@ void *Timsort(void *argument)
       if ( tsID >= nalloc )
 	{
 	  nalloc += NALLOC_INC;
-	  vdate = realloc(vdate, nalloc*sizeof(int));
-	  vtime = realloc(vtime, nalloc*sizeof(int));
-	  vars  = realloc(vars, nalloc*sizeof(field_t **));
+	  vdate = (int*) realloc(vdate, nalloc*sizeof(int));
+	  vtime = (int*) realloc(vtime, nalloc*sizeof(int));
+	  vars  = (field_t ***) realloc(vars, nalloc*sizeof(field_t **));
 	}
 
       vdate[tsID] = taxisInqVdate(taxisID1);
@@ -104,7 +99,7 @@ void *Timsort(void *argument)
 	  streamInqRecord(streamID1, &varID, &levelID);
 	  gridID   = vlistInqVarGrid(vlistID1, varID);
 	  gridsize = gridInqSize(gridID);
-	  vars[tsID][varID][levelID].ptr = malloc(gridsize*sizeof(double));
+	  vars[tsID][varID][levelID].ptr = (double*) malloc(gridsize*sizeof(double));
 	  streamReadRecord(streamID1, vars[tsID][varID][levelID].ptr, &nmiss);
 	  vars[tsID][varID][levelID].nmiss = nmiss;
 	}
@@ -114,9 +109,9 @@ void *Timsort(void *argument)
 
   nts = tsID;
 
-  sarray = malloc(ompNumThreads*sizeof(double *));
+  sarray = (double **) malloc(ompNumThreads*sizeof(double *));
   for ( i = 0; i < ompNumThreads; i++ )
-    sarray[i] = malloc(nts*sizeof(double));
+    sarray[i] = (double*) malloc(nts*sizeof(double));
 
   for ( varID = 0; varID < nvars; varID++ )
     {
@@ -128,15 +123,12 @@ void *Timsort(void *argument)
       for ( levelID = 0; levelID < nlevel; levelID++ )
 	{
 #if defined(_OPENMP)
-#pragma omp parallel for default(shared) private(i, ompthID, tsID)
+#pragma omp parallel for default(shared) private(i, tsID)
 #endif
 	  for ( i = 0; i < gridsize; i++ )
 	    {
-#if defined(_OPENMP)
-	      ompthID = omp_get_thread_num();
-#else
-	      ompthID = 0;
-#endif
+	      int ompthID = cdo_omp_get_thread_num();
+
 	      for ( tsID = 0; tsID < nts; tsID++ )
 		sarray[ompthID][tsID] = vars[tsID][varID][levelID].ptr[i];
 
