@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -35,6 +35,7 @@
 #include "pstream.h"
 #include "list.h"
 #include "grid.h"
+#include "constants.h"
 #include "stdnametable.h"
 
 
@@ -63,9 +64,8 @@
 #define T_DELTA          (75.0)
 #define SCALEHEIGHT   (10000.0)   /* [m] */
 #define P_ZERO         (1013.25)  /* surface pressure [hPa] */
-#define C_EARTH_GRAV      (9.80665)
-#define C_R             (287.05)  /*  specific gas constant for air */
-static double TMP4PRESSURE = (C_EARTH_GRAV*SCALEHEIGHT)/(C_R*T_ZERO);
+#define CC_R             (287.05)  /* specific gas constant for air */
+static double TMP4PRESSURE = (C_EARTH_GRAV*SCALEHEIGHT)/(CC_R*T_ZERO);
 
 static double
 std_atm_temperatur(double height)
@@ -127,8 +127,7 @@ void *Vargen(void *argument)
       gridID   = cdoDefineGrid(gridfile);
       if ( operatorArgc() == 2 )
         {
-          long idum;
-          idum = atol(operatorArgv()[1]);
+          int idum = parameter2int(operatorArgv()[1]);
           if ( idum >= 0 && idum < 0x7FFFFFFF )
             seed = idum;
         }
@@ -145,7 +144,7 @@ void *Vargen(void *argument)
     {
       operatorInputArg(cdoOperatorEnter(operatorID));
       operatorCheckArgc(2);
-      rconst   = atof(operatorArgv()[0]);
+      rconst   = parameter2double(operatorArgv()[0]);
       gridfile = operatorArgv()[1];
       gridID   = cdoDefineGrid(gridfile);
     }
@@ -172,10 +171,10 @@ void *Vargen(void *argument)
       if ( operatorArgc() < 2 ) cdoAbort("Too few arguments!");
       if ( operatorArgc() > 3 ) cdoAbort("Too many arguments!");
 
-      rstart = atof(operatorArgv()[0]);
-      rstop  = atof(operatorArgv()[1]);
+      rstart = parameter2double(operatorArgv()[0]);
+      rstop  = parameter2double(operatorArgv()[1]);
       if ( operatorArgc() == 3 )
-        rinc = atof(operatorArgv()[2]);
+        rinc = parameter2double(operatorArgv()[2]);
       else
         rinc = 1;
 
