@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -27,7 +27,7 @@
 #include "cdo.h"
 #include "cdo_int.h"
 #include "pstream.h"
-#include "vinterp.h"
+#include "after_vertint.h"
 
 
 #define R  287.07  /* spezielle Gaskonstante fuer Luft */
@@ -35,8 +35,8 @@
 
 void *Vertwind(void *argument)
 {
-  int streamID1, streamID2;
-  int vlistID1, vlistID2;
+  int streamID2;
+  int vlistID2;
   int taxisID1, taxisID2;
   int gridID, zaxisID, tsID;
   int nlevel, nrecs, recID, code;
@@ -45,7 +45,6 @@ void *Vertwind(void *argument)
   int gridsize, i;
   int offset;
   int nmiss, nmiss_out;
-  int ngp = 0, ngrids;
   int temp_code, sq_code, ps_code, omega_code, lsp_code;
   int tempID = -1, sqID = -1, psID = -1, omegaID = -1, lnpsID = -1;
   char varname[CDI_MAX_NAME];
@@ -58,31 +57,11 @@ void *Vertwind(void *argument)
 
   cdoInitialize(argument);
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
+  int vlistID1 = streamInqVlist(streamID1);
 
-  ngrids  = vlistNgrids(vlistID1);
-  for ( i = 0; i < ngrids; i++ )
-    {
-      gridID = vlistGrid(vlistID1, i);
-      if ( gridInqType(gridID) != GRID_SPECTRAL )
-	{
-	  ngp = gridInqSize(gridID);
-	  break;
-	}
-    }
-
-  /* check gridsize */
-  for ( i = 0; i < ngrids; i++ )
-    {
-      gridID = vlistGrid(vlistID1, i);
-      if ( gridInqType(gridID) != GRID_SPECTRAL )
-	{
-	  if ( ngp != gridInqSize(gridID) )
-	    cdoAbort("Grids have different size!");
-	}
-    }
+  int ngp = vlist_check_gridsize(vlistID1);
 
   temp_code  = 130;
   sq_code    = 133;
@@ -142,14 +121,14 @@ void *Vertwind(void *argument)
 
   gridsize = gridInqSize(gridID);
   nlevel = zaxisInqSize(zaxisID);
-  level  = malloc(nlevel*sizeof(double));
+  level  = (double*) malloc(nlevel*sizeof(double));
   zaxisInqLevels(zaxisID, level);
 
-  temp    = malloc(gridsize*nlevel*sizeof(double));
-  sq      = malloc(gridsize*nlevel*sizeof(double));
-  omega   = malloc(gridsize*nlevel*sizeof(double));
-  wms     = malloc(gridsize*nlevel*sizeof(double));
-  fpress  = malloc(gridsize*nlevel*sizeof(double));
+  temp    = (double*) malloc(gridsize*nlevel*sizeof(double));
+  sq      = (double*) malloc(gridsize*nlevel*sizeof(double));
+  omega   = (double*) malloc(gridsize*nlevel*sizeof(double));
+  wms     = (double*) malloc(gridsize*nlevel*sizeof(double));
+  fpress  = (double*) malloc(gridsize*nlevel*sizeof(double));
 
 
   if ( zaxisInqType(zaxisID) == ZAXIS_PRESSURE )
@@ -163,13 +142,13 @@ void *Vertwind(void *argument)
     }
   else if ( zaxisInqType(zaxisID) == ZAXIS_HYBRID )
     {
-      ps_prog = malloc(gridsize*sizeof(double));
-      hpress  = malloc(gridsize*(nlevel+1)*sizeof(double));
+      ps_prog = (double*) malloc(gridsize*sizeof(double));
+      hpress  = (double*) malloc(gridsize*(nlevel+1)*sizeof(double));
   
       nvct = zaxisInqVctSize(zaxisID);
       if ( nlevel == (nvct/2 - 1) )
 	{
-	  vct = malloc(nvct*sizeof(double));
+	  vct = (double*) malloc(nvct*sizeof(double));
 	  zaxisInqVct(zaxisID, vct);
 	}
       else
