@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -28,12 +28,21 @@
 #include <math.h>
 #include <float.h>
 
+#include "compare.h"
 #include "timebase.h"
 #include "field.h"
 #include "functs.h"
 #include "dmemory.h"
 #include "process.h"
 #include "const.h"
+#include "util.h"
+#include "datetime.h"
+
+#define  OPENMP4  201307
+#if defined(_OPENMP) && defined(OPENMP4) && _OPENMP >= OPENMP4
+#define  HAVE_OPENMP4  1
+#endif
+
 
 #ifndef strdupx
 #ifndef strdup
@@ -52,37 +61,17 @@ char *strdup(const char *s);
 #endif
 
 
+#define  cmpstr(s1, s2)          (strncmp(s1, s2, strlen(s2)))
+#define  cmpstrlen(s1, s2, len)  (strncmp(s1, s2, len = strlen(s2)))
+
+
 /* sxxxYYYYMMDDhhmm0 */
 #define  DATE_LEN  31        /* YYYYMMDDhhmmss allocate DTLEN+1 !!!! */
 #define  SET_DATE(dtstr, date, time)      (sprintf(dtstr, "%*d%*d", DATE_LEN-6, date, 6, time))
 #define  DATE_IS_NEQ(dtstr1, dtstr2, len) (memcmp(dtstr1, dtstr2, len) != 0)
 
-
-#if defined(__xlC__) /* performance problems on IBM */
-#ifndef DBL_IS_NAN
-#  define DBL_IS_NAN(x)     ((x) != (x))
-#endif
-#else
-#ifndef DBL_IS_NAN
-#if defined(HAVE_DECL_ISNAN)
-#  define DBL_IS_NAN(x)     (isnan(x))
-#elif defined(FP_NAN)
-#  define DBL_IS_NAN(x)     (fpclassify(x) == FP_NAN)
-#else
-#  define DBL_IS_NAN(x)     ((x) != (x))
-#endif
-#endif
-#endif
-
-#ifndef DBL_IS_EQUAL
-/*#define DBL_IS_EQUAL(x,y) (!(x < y || y < x)) */
-#  define DBL_IS_EQUAL(x,y) (DBL_IS_NAN(x)||DBL_IS_NAN(y)?(DBL_IS_NAN(x)&&DBL_IS_NAN(y)?1:0):!(x < y || y < x))
-#endif
-
-#ifndef IS_EQUAL
-#  define IS_NOT_EQUAL(x,y) (x < y || y < x)
-#  define IS_EQUAL(x,y)     (!IS_NOT_EQUAL(x,y))
-#endif
+enum T_WEIGHT_MODE {WEIGHT_OFF, WEIGHT_ON};
+enum T_EIGEN_MODE  {JACOBI, DANIELSON_LANCZOS};
 
 
 #ifndef  M_LN10
@@ -102,11 +91,6 @@ char *strdup(const char *s);
 #define  CDO_EXP_LOCAL   1
 #define  CDO_EXP_REMOTE  2
 
-
-enum {DATE_FIRST, DATE_LAST, DATE_MIDDLE};
-
-void strtolower(char *str);
-
 void print_pthread_info(void);
 
 void cdoProcessTime(double *utime, double *stime);
@@ -116,8 +100,6 @@ char   *commandLine(void);
 int     readline(FILE *fp, char *line, int len);
 
 int zaxis2ltype(int zaxisID);
-int ztype2ltype(int zaxistype);
-int ltype2ztype(int ltype);
 
 
 int nfc2nlat(int nfc, int ntr);
@@ -128,42 +110,13 @@ int ntr2nlat_linear(int ntr);
 int compNlon(int nlat);
 
 void param2str(int param, char *paramstr, int maxlen);
+void datetime2str(int date, int time, char *datetimestr, int maxlen);
 void date2str(int date, char *datestr, int maxlen);
 void time2str(int time, char *timestr, int maxlen);
 
 const char * tunit2str(int tunits);
 const char * calendar2str(int calendar);
 
-
-typedef struct {
-  int   date;
-  int   time;
-} datetime_t;
-
-typedef struct
-{
-  datetime_t v;
-  datetime_t b[2];
-} dtinfo_t;
-
-typedef struct {
-  int   julday;
-  int   secofday;
-} juldate_t;
-
-
-juldate_t juldate_encode(int calendar, int date, int time);
-void      juldate_decode(int calendar, juldate_t juldate, int *date, int *time);
-juldate_t juldate_sub(juldate_t juldate2, juldate_t juldate1);
-juldate_t juldate_add_seconds(int seconds, juldate_t juldate);
-double    juldate_to_seconds(juldate_t juldate);
-
-void    get_timestat_date(int *tstat_date);
-void    datetime_avg(int dpy, int ndates, datetime_t *datetime);
-void    datetime_avg_dtinfo(int dpy, int ndates, dtinfo_t *dtinfo);
-void    taxisInqDTinfo(int taxisID, dtinfo_t *dtinfo);
-void    taxisDefDTinfo(int taxisID, dtinfo_t dtinfo);
-
 int     days_per_month(int calendar, int year, int month);
 int     days_per_year(int calendar, int year);
 int     calendar_dpy(int calendar);
@@ -191,5 +144,11 @@ void minmaxval(long nvals, double *array, int *imiss, double *minval, double *ma
 
 off_t filesize(const char *filename);
 
+char *expand_filename(const char *string);
+
+double parameter2double(const char *string);
+int    parameter2int(const char *string);
+int    parameter2intlist(const char *string);
+
 
 #endif  /* _CDO_INT_H */
