@@ -41,49 +41,41 @@
 
 #include <stdlib.h>
 
-#ifdef __WITHOUT_ISO_C_BINDING
-#include "cfortran.h"
-#endif
-
 /**
  * gives a unique index for a given pointer
  * @param[in] pointer
  * @return unique value associated to pointer
  * @see unique_id_to_pointer
  */
-unsigned pointer_to_unique_id(void * pointer);
+unsigned yac_pointer_to_unique_id(void * pointer);
 
 /**
  * gives the pointer that is associated to the given id
  * @param[in] id unique index previously returned by pointer_to_unique_id
  * @return pointer the is associated to the given id \n NULL if the id is invalid
  */
-void *   unique_id_to_pointer(unsigned id);
+void * yac_unique_id_to_pointer(unsigned id);
 
 /**
  * frees all memory used for the pointer/unique_id conversion
  * \remarks this should only be called after the last call to 
- *          \ref pointer_to_unique_id and \ref unique_id_to_pointer, because afterwords
- *          \ref unique_id_to_pointer will not be able to return the respective pointers
+ *          \ref yac_pointer_to_unique_id and \ref yac_unique_id_to_pointer, because afterwards
+ *          \ref yac_unique_id_to_pointer will not be able to return the respective pointers
  *          for previously valid unique ids
  */
-void free_pointer_unique_lookup();
+void yac_free_pointer_unique_lookup();
 
 /**
  * prints a short error message and info from where it was called
  * followed by an exit. 
  */
-void abort_message ( char * text, char * file, int line );
-
-#ifdef __WITHOUT_ISO_C_BINDING
-     FCALLSCSUB3 ( yac_abort_message, YAC_ABORT_MESSAGE, yac_abort_message, STRING, STRING, PINT )
-#endif
+void yac_internal_abort_message ( char * text, char * file, int line );
 
 /** \example test_quicksort.c
  * This contains an example of how to use quicksort_index.
  */
 
-void quicksort_index ( int * a, int n, int * idx);
+void yac_quicksort_index ( int * a, int n, int * idx);
 
 /** \example test_mergesort.c
  *
@@ -97,16 +89,72 @@ void yac_mergesort(void* base, size_t num, size_t size,
  *
  * Hash function
  *
- * This algorithm (k=33) was first reported by dan bernstein many
+ * This algorithm (k=33) was first reported by Dan Bernstein many
  * years ago in comp.lang.c. another version of this algorithm (now
- * favored by bernstein) uses xor: hash(i) = hash(i - 1) * 33 ^
+ * favored by Bernstein) uses xor: hash(i) = hash(i - 1) * 33 ^
  * str[i]; the magic of number 33 (why it works better than many other
  * constants, prime or not) has never been adequately explained.
  *
  * Source: http://www.cse.yorku.ca/~oz/hash.html 
  */
 
-unsigned int hash(const char *str);
+unsigned int yac_hash(const char *str);
+
+/**
+ * remove duplicated entries from a list of integers
+ * @param[in,out] array array containing a sorted (ascending) list of integers
+ * @param[in,out] n     number of entries in array
+ */
+static inline void yac_remove_duplicates_int(int * array, unsigned * n) {
+
+   unsigned const N = *n;
+   unsigned pos = 0;
+
+   if (N == 0) return;
+
+   int prev = array[0];
+
+   for (unsigned i = 1; i < N; ++i) {
+
+      if (array[i] == prev) continue;
+
+      prev = array[i];
+      ++pos;
+
+      if (pos != i)
+         array[pos] = array[i];
+   }
+
+   *n = pos + 1;
+}
+
+/**
+ * remove duplicated entries from a list of integers
+ * @param[in,out] array array containing a sorted (ascending) list of integers
+ * @param[in,out] n     number of entries in array
+ */
+static inline void yac_remove_duplicates_uint(unsigned * array, unsigned * n) {
+
+   unsigned const N = *n;
+   unsigned pos = 0;
+
+   if (N == 0) return;
+
+   unsigned prev = array[0];
+
+   for (unsigned i = 1; i < N; ++i) {
+
+      if (array[i] == prev) continue;
+
+      prev = array[i];
+      ++pos;
+
+      if (pos != i)
+         array[pos] = array[i];
+   }
+
+   *n = pos + 1;
+}
 
 /* =======================================================================
    Macros
