@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -422,7 +422,7 @@ void zonpctl(field_t field1, field_t *field2, int p)
   
   if ( nmiss > 0 )
     {
-      array2 = malloc(nx*sizeof(double));
+      array2 = (double*) malloc(nx*sizeof(double));
       
       for ( j = 0; j < ny; j++ )
         {
