@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -51,15 +51,6 @@ int readline(FILE *fp, char *line, int len);
 #define  MIN(a,b)  ((a) < (b) ? (a) : (b))
 #undef   MAX
 #define  MAX(a,b)  ((a) > (b) ? (a) : (b))
-#undef   NINT
-#define  NINT(x)   ((x) < 0 ? (int)((x)-0.5) : (int)((x)+0.5))
-
-struct PGMSTAT
-{
-  int intract;
-};
-
-struct PGMSTAT pgmstat;
 
 
 static void namelist_init(namelist_t *namelist, const char *name)
@@ -74,7 +65,7 @@ namelist_t *namelistNew(const char *name)
 {
   namelist_t *namelist;
 
-  namelist = malloc(sizeof(namelist_t));
+  namelist = (namelist_t*) malloc(sizeof(namelist_t));
 
   namelist_init(namelist, name);
 
@@ -190,7 +181,7 @@ int namelistAdd(namelist_t *nml, const char *name, int type, int dis, void *ptr,
       return (-1);
     }
 
-  nml_entry = malloc(sizeof(nml_entry_t));
+  nml_entry = (nml_entry_t*) malloc(sizeof(nml_entry_t));
 
   nml_entry->name = strdup(name);
   nml_entry->type = type;
@@ -229,8 +220,8 @@ int namelistNum(namelist_t *nml, const char *name)
   return (nocc);
 }
 
-
-static void getnite(FILE *nmlfp, namelist_t *nml)
+static
+void getnite(FILE *nmlfp, namelist_t *nml)
 {
   int nst, i, j;
   int linelen;
@@ -241,27 +232,27 @@ static void getnite(FILE *nmlfp, namelist_t *nml)
     {
       for ( i = nst; i < MAX_LINE_LEN; i++ )
 	{
-	  if ( nml->line.linelc[i] == 0 ) break;
+	  char *linelc = nml->line.linelc;
+	  int ch = (int) linelc[i];
 
-          if      (   nml->line.linelc[i] == ' ' ) continue;
-	  else if (   nml->line.linelc[i] == '=' ) continue;
-          else if (   nml->line.linelc[i] == ',' ) continue;
-          else if ( ((nml->line.linelc[i] >= 'a')  &&
-		     (nml->line.linelc[i] <= 'z')) ||
-	             (nml->line.linelc[i] == '_')  ||
-	             (nml->line.linelc[i] == '/')  ||
-	             (nml->line.linelc[i] == '$')  ||
-		     (nml->line.linelc[i] == '&') )
+	  if ( ch == 0 ) break;
+
+          if      (   ch == ' ' ) continue;
+	  else if (   ch == '=' ) continue;
+          else if (   ch == ',' ) continue;
+          else if ( ((ch >= 'a')  && (ch <= 'z')) ||
+	             (ch == '_')  || (ch == '/')  ||
+	             (ch == '$')  || (ch == '&') )
 	    {
 	      nml->line.nptype = NML_KEYWORD;
 	      nml->line.namitf = i;
 	      for ( j = nml->line.namitf+1; j < MAX_LINE_LEN; j++ )
 		{
-		  if ( !(islower((int) nml->line.linelc[j]) ||
-			 (((int) nml->line.linelc[j]) == '_') ||
-			 (((int) nml->line.linelc[j]) == '-') ||
-			 (((int) nml->line.linelc[j]) == '+') ||
-			 isdigit((int) nml->line.linelc[j])) )
+		  if ( !(islower((int) linelc[j]) ||
+			 (((int) linelc[j]) == '_') ||
+			 (((int) linelc[j]) == '-') ||
+			 (((int) linelc[j]) == '+') ||
+			 isdigit((int) linelc[j])) )
 		    {
 		      nml->line.namitl = j - 1;
 		      return;
@@ -270,14 +261,14 @@ static void getnite(FILE *nmlfp, namelist_t *nml)
 	      nml->line.namitl = MAX_LINE_LEN;
 	      return;
 	    }
-          else if ( nml->line.linelc[i] == '\'' ||
-		    nml->line.linelc[i] == '\"' ||
-		    nml->line.linelc[i] == '`')
+          else if ( ch == '\'' ||
+		    ch == '\"' ||
+		    ch == '`')
 	    {
 	      nml->line.nptype = NML_TEXT;
 	      nml->line.namitf = i;
 	      for ( j = nml->line.namitf+1; j < MAX_LINE_LEN; j++ )
-		if (nml->line.linelc[j] == nml->line.linelc[nml->line.namitf])
+		if ( linelc[j] == linelc[nml->line.namitf])
 		  {
  		    nml->line.namitl = j;
 		    return;
@@ -285,21 +276,18 @@ static void getnite(FILE *nmlfp, namelist_t *nml)
 	      nml->line.namitl = MAX_LINE_LEN + 1;
 	      return;
 	    }
-          else if ( (nml->line.linelc[i] >= '0'  &&
-		     nml->line.linelc[i] <= '9') ||
-		     nml->line.linelc[i] == '+'  ||
-		     nml->line.linelc[i] == '-'  ||
-		     nml->line.linelc[i] == '.' )
+          else if ( (ch >= '0'  && ch <= '9') ||
+		     ch == '+'  || ch == '-'  || ch == '.' )
 	    {
 	      nml->line.nptype = NML_NUMBER;
 	      nml->line.namitf = i;
 	      for ( j = i+1; j < MAX_LINE_LEN; j++)
 		{
-		  if ( nml->line.linelc[j] >= '0' && nml->line.linelc[j] <= '9' ) continue;
-	          else if ( nml->line.linelc[j] == '+' ) continue;
-		  else if ( nml->line.linelc[j] == '-' ) continue;
-         	  else if ( nml->line.linelc[j] == '.' ) continue;
-		  else if ( nml->line.linelc[j] == 'e' ) continue;
+		  if ( linelc[j] >= '0' && linelc[j] <= '9' ) continue;
+	          else if ( linelc[j] == '+' ) continue;
+		  else if ( linelc[j] == '-' ) continue;
+         	  else if ( linelc[j] == '.' ) continue;
+		  else if ( linelc[j] == 'e' ) continue;
                   else
 		    {
 		      nml->line.namitl = j - 1;
@@ -326,8 +314,8 @@ static void getnite(FILE *nmlfp, namelist_t *nml)
   nml->line.nptype = NML_NIX;
 }
 
-
-static void rdnlsgl(namelist_t *nml, void *var, int ntyp, int nlen, int *nocc)
+static
+void rdnlsgl(namelist_t *nml, void *var, int ntyp, int nlen, int *nocc)
 {
   if ( nml->line.nptype == NML_NUMBER )
     {
@@ -346,6 +334,19 @@ static void rdnlsgl(namelist_t *nml, void *var, int ntyp, int nlen, int *nocc)
 	  ((double *)var)[*nocc] = atof(&nml->line.lineac[nml->line.namitf]);
           *nocc += 1;
 	}
+      else if ( ntyp == NML_WORD )
+	{
+	  int i, len;
+
+	  if ( *nocc < nlen )
+	    {
+	      len = nml->line.namitl - nml->line.namitf + 1;
+	      ((char **)var)[*nocc] = (char*) calloc((size_t)len+1, sizeof(char));
+	      for ( i = 0; i < len; i++ )
+		((char **)var)[*nocc][i] = nml->line.lineac[nml->line.namitf+i];
+	      *nocc += 1;
+	    }
+	}
       else
 	{
           nml->line.nptype = func_2;
@@ -383,7 +384,7 @@ static void rdnlsgl(namelist_t *nml, void *var, int ntyp, int nlen, int *nocc)
       if ( *nocc < nlen )
 	{
 	  len = nml->line.namitl - nml->line.namitf + 1;
-	  ((char **)var)[*nocc] = calloc((size_t)len+1, sizeof(char));
+	  ((char **)var)[*nocc] = (char*) calloc((size_t)len+1, sizeof(char));
 	  for ( i = 0; i < len; i++ )
 	    ((char **)var)[*nocc][i] = nml->line.lineac[nml->line.namitf+i];
 	  *nocc += 1;
@@ -533,12 +534,6 @@ void namelistRead(FILE *nmlfp, namelist_t *nml)
 	  printf(" * valid parameters and values specified so far are\n");
 
 	  nml_print(nml, PRINT_ALL);
-
-          if ( ! pgmstat.intract )
-	    {
-	      fprintf(stderr, "Namelist error!\n");
-	      return;
-	    }
         }
       else
 	{
@@ -570,8 +565,6 @@ void namelistRead(FILE *nmlfp, namelist_t *nml)
 	  printf(" * valid parameters and values specified so far are\n");
 
 	  nml_print(nml, PRINT_ALL);
-
-	  if ( ! pgmstat.intract ) fprintf(stderr, "Namelist error!\n");
         }
     }
 
