@@ -17,6 +17,8 @@ stdnametable_t;
 
 const stdnametable_t stdnametable[] = {
   /* varid                       code    name                standard name                 units */
+  { air_pressure,                   1,  "apres",            "air_pressure",               "Pa" },
+  { pressure_thickness,             2,  "dpress",           "pressure_thickness",         "Pa" },
   { surface_geopotential,         129,  "geosp",            "surface_geopotential",       "m2 s-2" },
   { geopotential,                 129,  "z",                "geopotential",               "m2 s-2" },
   { air_temperature,              130,  "ta",               "air_temperature",            "K" },
